@@ -331,6 +331,7 @@ class Maintenance(salt.utils.process.SignalHandlingMultiprocessingProcess):
                 self.loop_interval = self.schedule.loop_interval
         except Exception as exc:
             log.error('Exception %s occurred in scheduled job', exc)
+        self.schedule.cleanup_subprocesses()
 
     def handle_presence(self, old_present):
         '''
@@ -676,7 +677,7 @@ class Master(SMaster):
             pub_channels = []
             log.info('Creating master publisher process')
             log_queue = salt.log.setup.get_multiprocessing_logging_queue()
-            for transport, opts in iter_transport_opts(self.opts):
+            for _, opts in iter_transport_opts(self.opts):
                 chan = salt.transport.server.PubServerChannel.factory(opts)
                 chan.pre_fork(self.process_manager, kwargs={'log_queue': log_queue})
                 pub_channels.append(chan)
