@@ -42,10 +42,13 @@ Connection module for Amazon SNS
 # keep lint from choking on _get_conn and _cache_id
 #pylint: disable=E0602
 
-from __future__ import absolute_import
-
+# Import Python libs
+from __future__ import absolute_import, print_function, unicode_literals
 import logging
 
+# Import Salt libs
+import salt.utils.versions
+
 log = logging.getLogger(__name__)
 
 # Import third party libs
@@ -64,10 +67,12 @@ def __virtual__():
     '''
     Only load if boto libraries exist.
     '''
-    if not HAS_BOTO:
-        return (False, 'The boto_sns module could not be loaded: boto libraries not found')
-    __utils__['boto.assign_funcs'](__name__, 'sns', pack=__salt__)
-    return True
+    has_boto_reqs = salt.utils.versions.check_boto_reqs(
+        check_boto3=False
+    )
+    if has_boto_reqs is True:
+        __utils__['boto.assign_funcs'](__name__, 'sns', pack=__salt__)
+    return has_boto_reqs
 
 
 def get_all_topics(region=None, key=None, keyid=None, profile=None):
@@ -120,7 +125,7 @@ def create(name, region=None, key=None, keyid=None, profile=None):
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
     conn.create_topic(name)
-    log.info('Created SNS topic {0}'.format(name))
+    log.info('Created SNS topic %s', name)
     _invalidate_cache()
     return True
 
@@ -135,7 +140,7 @@ def delete(name, region=None, key=None, keyid=None, profile=None):
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
     conn.delete_topic(get_arn(name, region, key, keyid, profile))
-    log.info('Deleted SNS topic {0}'.format(name))
+    log.info('Deleted SNS topic %s', name)
     _invalidate_cache()
     return True
 
@@ -170,7 +175,7 @@ def subscribe(topic, protocol, endpoint, region=None, key=None, keyid=None, prof
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
     conn.subscribe(get_arn(topic, region, key, keyid, profile), protocol, endpoint)
-    log.info('Subscribe {0} {1} to {2} topic'.format(protocol, endpoint, topic))
+    log.info('Subscribe %s %s to %s topic', protocol, endpoint, topic)
     try:
         del __context__[_subscriptions_cache_key(topic)]
     except KeyError:
@@ -197,9 +202,9 @@ def unsubscribe(topic, subscription_arn, region=None, key=None, keyid=None, prof
 
     try:
         conn.unsubscribe(subscription_arn)
-        log.info('Unsubscribe {0} to {1} topic'.format(subscription_arn, topic))
+        log.info('Unsubscribe %s to %s topic', subscription_arn, topic)
     except Exception as e:
-        log.error('Unsubscribe Error: {0}'.format(e))
+        log.error('Unsubscribe Error', exc_info=True)
         return False
     else:
         __context__.pop(_subscriptions_cache_key(topic), None)
