@@ -14,11 +14,11 @@ import logging
 # Import Salt libs
 import salt.utils.platform
 import salt.utils.win_functions
+import salt.utils.winapi
 
 try:
     import win32api
     import win32com.client
-    import pythoncom
     import pywintypes
     HAS_DEPENDENCIES = True
 except ImportError:
@@ -46,8 +46,8 @@ def _get_computer_object():
     Returns:
         object: Returns the computer object for the local machine
     '''
-    pythoncom.CoInitialize()
-    nt = win32com.client.Dispatch('AdsNameSpaces')
+    with salt.utils.winapi.Com():
+        nt = win32com.client.Dispatch('AdsNameSpaces')
     return nt.GetObject('', 'WinNT://.,computer')
 
 
@@ -62,8 +62,8 @@ def _get_group_object(name):
     Returns:
         object: The specified group object
     '''
-    pythoncom.CoInitialize()
-    nt = win32com.client.Dispatch('AdsNameSpaces')
+    with salt.utils.winapi.Com():
+        nt = win32com.client.Dispatch('AdsNameSpaces')
     return nt.GetObject('', 'WinNT://./' + name + ',group')
 
 
@@ -75,8 +75,8 @@ def _get_all_groups():
     Returns:
         iter: A list of objects for all groups on the machine
     '''
-    pythoncom.CoInitialize()
-    nt = win32com.client.Dispatch('AdsNameSpaces')
+    with salt.utils.winapi.Com():
+        nt = win32com.client.Dispatch('AdsNameSpaces')
     results = nt.GetObject('', 'WinNT://.')
     results.Filter = ['group']
     return results
