@@ -1,7 +1,10 @@
 # -*- coding: utf-8 -*-
 '''
-Hjson Renderer for Salt
-http://laktak.github.io/hjson/
+hjson renderer for Salt
+
+See the hjson_ documentation for more information
+
+.. _hjson: http://laktak.github.io/hjson/
 '''
 
 from __future__ import absolute_import, print_function, unicode_literals
