@@ -4,13 +4,13 @@ Manage GlusterFS pool.
 '''
 
 # Import python libs
-from __future__ import generators
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, \
+    print_function, generators
 import logging
-import socket
 
 # Import salt libs
 import salt.utils.cloud as suc
+import salt.utils.network
 from salt.exceptions import SaltCloudException
 
 log = logging.getLogger(__name__)
@@ -65,15 +65,21 @@ def peered(name):
 
     try:
         suc.check_name(name, 'a-zA-Z0-9._-')
-    except SaltCloudException as e:
+    except SaltCloudException:
         ret['comment'] = 'Invalid characters in peer name.'
         return ret
 
-    # Check if the name resolves to localhost
-    if socket.gethostbyname(name) in __salt__['network.ip_addrs']():
-        ret['result'] = True
-        ret['comment'] = 'Peering with localhost is not needed'
-        return ret
+    # Check if the name resolves to one of this minion IP addresses
+    name_ips = salt.utils.network.host_to_ips(name)
+    if name_ips is not None:
+        # if it is None, it means resolution fails, let's not hide
+        # it from the user.
+        this_ips = set(salt.utils.network.ip_addrs())
+        this_ips.update(salt.utils.network.ip_addrs6())
+        if this_ips.intersection(name_ips):
+            ret['result'] = True
+            ret['comment'] = 'Peering with localhost is not needed'
+            return ret
 
     peers = __salt__['glusterfs.peer_status']()
 
@@ -87,8 +93,7 @@ def peered(name):
         ret['result'] = None
         return ret
 
-    peered = __salt__['glusterfs.peer'](name)
-    if not peered:
+    if not __salt__['glusterfs.peer'](name):
         ret['comment'] = 'Failed to peer with {0}, please check logs for errors'.format(name)
         return ret
 
@@ -104,7 +109,7 @@ def peered(name):
 
 
 def volume_present(name, bricks, stripe=False, replica=False, device_vg=False,
-            transport='tcp', start=False, force=False):
+                   transport='tcp', start=False, force=False, arbiter=False):
     '''
     Ensure that the volume exists
 
@@ -114,25 +119,45 @@ def volume_present(name, bricks, stripe=False, replica=False, device_vg=False,
     bricks
         list of brick paths
 
+    replica
+        replica count for volume
+
+    arbiter
+        use every third brick as arbiter (metadata only)
+
+        .. versionadded:: 2019.2.0
+
     start
         ensure that the volume is also started
 
     .. code-block:: yaml
 
         myvolume:
-          glusterfs.created:
+          glusterfs.volume_present:
             - bricks:
                 - host1:/srv/gluster/drive1
                 - host2:/srv/gluster/drive2
 
         Replicated Volume:
-          glusterfs.created:
+          glusterfs.volume_present:
             - name: volume2
             - bricks:
               - host1:/srv/gluster/drive2
               - host2:/srv/gluster/drive3
             - replica: 2
             - start: True
+
+        Replicated Volume with arbiter brick:
+          glusterfs.volume_present:
+            - name: volume3
+            - bricks:
+              - host1:/srv/gluster/drive2
+              - host2:/srv/gluster/drive3
+              - host3:/srv/gluster/drive4
+            - replica: 3
+            - arbiter: True
+            - start: True
+
     '''
     ret = {'name': name,
            'changes': {},
@@ -153,9 +178,10 @@ def volume_present(name, bricks, stripe=False, replica=False, device_vg=False,
             ret['result'] = None
             return ret
 
-        vol_created = __salt__['glusterfs.create_volume'](name, bricks, stripe,
-                                                  replica, device_vg,
-                                                  transport, start, force)
+        vol_created = __salt__['glusterfs.create_volume'](
+            name, bricks, stripe,
+            replica, device_vg,
+            transport, start, force, arbiter)
 
         if not vol_created:
             ret['comment'] = 'Creation of volume {0} failed'.format(name)
@@ -295,3 +321,111 @@ def add_volume_bricks(name, bricks):
 
     ret['comment'] = 'Adding bricks to volume {0} failed'.format(name)
     return ret
+
+
+def op_version(name, version):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Add brick(s) to an existing volume
+
+    name
+        Volume name
+
+    version
+        Version to which the cluster.op-version should be set
+
+    .. code-block:: yaml
+
+        myvolume:
+          glusterfs.op_version:
+            - name: volume1
+            - version: 30707
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'comment': '',
+           'result': False}
+
+    try:
+        current = int(__salt__['glusterfs.get_op_version'](name))
+    except TypeError:
+        ret['result'] = False
+        ret['comment'] = __salt__['glusterfs.get_op_version'](name)[1]
+        return ret
+
+    if current == version:
+        ret['comment'] = 'Glusterfs cluster.op-version for {0} already set to {1}'.format(name, version)
+        ret['result'] = True
+        return ret
+    elif __opts__['test']:
+        ret['comment'] = 'An attempt would be made to set the cluster.op-version for {0} to {1}.'.format(name, version)
+        ret['result'] = None
+        return ret
+
+    result = __salt__['glusterfs.set_op_version'](version)
+
+    if result[0] is False:
+        ret['comment'] = result[1]
+        return ret
+
+    ret['comment'] = result
+    ret['changes'] = {'old': current, 'new': version}
+    ret['result'] = True
+    return ret
+
+
+def max_op_version(name):
+    '''
+    .. versionadded:: 2019.2.0
+
+    Add brick(s) to an existing volume
+
+    name
+        Volume name
+
+    .. code-block:: yaml
+
+        myvolume:
+          glusterfs.max_op_version:
+            - name: volume1
+            - version: 30707
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'comment': '',
+           'result': False}
+
+    try:
+        current = int(__salt__['glusterfs.get_op_version'](name))
+    except TypeError:
+        ret['result'] = False
+        ret['comment'] = __salt__['glusterfs.get_op_version'](name)[1]
+        return ret
+
+    try:
+        max_version = int(__salt__['glusterfs.get_max_op_version']())
+    except TypeError:
+        ret['result'] = False
+        ret['comment'] = __salt__['glusterfs.get_max_op_version']()[1]
+        return ret
+
+    if current == max_version:
+        ret['comment'] = 'The cluster.op-version is already set to the cluster.max-op-version of {0}'.format(current)
+        ret['result'] = True
+        return ret
+    elif __opts__['test']:
+        ret['comment'] = 'An attempt would be made to set the cluster.op-version to {0}.'.format(max_version)
+        ret['result'] = None
+        return ret
+
+    result = __salt__['glusterfs.set_op_version'](max_version)
+
+    if result[0] is False:
+        ret['comment'] = result[1]
+        return ret
+
+    ret['comment'] = result
+    ret['changes'] = {'old': current, 'new': max_version}
+    ret['result'] = True
+    return ret
