@@ -128,6 +128,7 @@ def state(name,
         queue=False,
         subset=None,
         orchestration_jid=None,
+        failhard=None,
         **kwargs):
     '''
     Invoke a state run on a given target
@@ -221,6 +222,11 @@ def state(name,
 
         .. versionadded:: 2017.7.0
 
+    failhard
+        pass failhard down to the executing state
+
+        .. versionadded:: 2019.2.2
+
     Examples:
 
     Run a list of sls files via :py:func:`state.sls <salt.state.sls>` on target
@@ -311,9 +317,13 @@ def state(name,
 
     if batch is not None:
         cmd_kw['batch'] = six.text_type(batch)
+
     if subset is not None:
         cmd_kw['subset'] = subset
 
+    if failhard is True or __opts__.get('failhard'):
+        cmd_kw['failhard'] = True
+
     masterless = __opts__['__role'] == 'minion' and \
                  __opts__['file_client'] == 'local'
     if not masterless:
@@ -428,6 +438,7 @@ def function(
         timeout=None,
         batch=None,
         subset=None,
+        failhard=None,
         **kwargs):  # pylint: disable=unused-argument
     '''
     Execute a single module function on a remote minion via salt or salt-ssh
@@ -477,6 +488,11 @@ def function(
 
         .. versionadded:: 2017.7.0
 
+    failhard
+        pass failhard down to the executing state
+
+        .. versionadded:: 2019.2.2
+
     '''
     func_ret = {'name': name,
                 'changes': {},
@@ -502,6 +518,9 @@ def function(
     cmd_kw['expect_minions'] = expect_minions
     cmd_kw['_cmd_meta'] = True
 
+    if failhard is True or __opts__.get('failhard'):
+        cmd_kw['failhard'] = True
+
     if ret_config:
         cmd_kw['ret_config'] = ret_config
 
