@@ -127,7 +127,6 @@ log = logging.getLogger(__name__)
 
 __func_alias__ = {
     'import_': 'import',
-    'export_': 'export',
 }
 
 # Define the state's virtual name
@@ -266,16 +265,17 @@ def resource_present(name, resource_type, resource_selector_property, resource_s
         unique resource identifier
     resource_selector_value : string
         value for resource selection
-    **kwargs : string|int|...
+    kwargs : string|int|...
         resource properties
 
     .. warning::
-        Both resource_selector_property and resource_selector_value must be provided, some properties
-        like ```name``` are already reserved by salt in there states.
+        Both resource_selector_property and resource_selector_value must be
+        provided, some properties like ``name`` are already reserved by salt in
+        states.
 
     .. note::
-        You can set both resource_selector_property and resource_selector_value to None for
-        resources that do not require them.
+        You can set both resource_selector_property and resource_selector_value
+        to None for resources that do not require them.
 
     '''
     ret = {'name': name,
@@ -622,7 +622,7 @@ def halted(name, graceful=True):
     return ret
 
 
-def export_(name, path, replace=False):
+def export(name, path, replace=False):
     '''
     Export a zones configuration
 
@@ -759,6 +759,11 @@ def import_(name, path, mode='import', nodataset=False, brand_opts=None):
         ``install``: will import and then try to install the zone
         ``attach``: will import and then try to attach of the zone
 
+    .. code-block:: yaml
+
+        omipkg1:
+          zone.import:
+            - path: /foo/bar/baz
     '''
     ret = {'name': name,
            'changes': {},
